<?php

/**
 * @file
 * Controls the server monitor administration screens.
 */

/**
 * System settings form for the Server Monitor module.
 */
function server_monitor_admin_form() {
  $form = array();
  $form['general_options'] = array(
    '#type' => 'fieldset',
    '#title' => t('General Configuration'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $last_run = variable_get('server_monitor_last_run', 0) ?
    date(SERVER_MONITOR_DATE_FORMAT, variable_get('server_monitor_last_run', 0)) :
    'Never';
  $form['general_options']['server_monitor_cron'] = array(
    '#type' => 'checkbox',
    '#title' => t('Check servers status when cron runs?'),
    '#description' => t('Last run: @date', array('@date' => $last_run)),
    '#default_value' => variable_get('server_monitor_cron', 1),
  );
  $form['general_options']['server_monitor_run'] = array(
    '#type' => 'submit',
    '#value' => t('Run monitor'),
  );
  $form['general_options']['server_monitor_use_json_block'] = array(
    '#type' => 'checkbox',
    '#title' => t('Use JSON generated block?'),
    '#description' => t('In order to prevent caching of stale monitor results you can use the more dynamic JSON generated result block.<br /><em>Warning:</em> This method results in an additional file (!file) being loaded.',
      array(
        '!file' => l(t('status.json'), 'server_monitor/status.json', array(
          'attributes' => array('target' => '_blank'),
        )),
      )),
    '#default_value' => variable_get('server_monitor_use_json_block', 0),
  );
  $form['servers'] = array(
    '#type' => 'fieldset',
    '#title' => t('Current server list'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['servers']['list']['#tree'] = TRUE;
  $stmt = "SELECT * FROM {server_monitor} ORDER BY weight ASC";
  $result = db_query($stmt);
  $row_knt = 0;
  $servers = array();
  $status = array(-1 => t('N/A'), 0 => t('Down'), 1 => 'Up');
  while ($row = db_fetch_array($result)) {
    $action = l(t('edit'), 'admin/settings/server_monitor/edit/' . $row['sid']);
    if (user_access('delete server monitor')) {
      $action .= '&nbsp;|&nbsp;';
      $action .= l(t('delete'), 'admin/settings/server_monitor/delete/' . $row['sid']);
    }
    $last_checked = $row['timestamp'] ?
      t('Last checked on @timestamp', array(
        '@timestamp' => date(SERVER_MONITOR_DATE_FORMAT, $row['timestamp']),
      )) : 'Never checked';
    $form['servers']['list'][$row['sid']] = array(
      'name' => array('#value' => $row['name']),
      'address' => array(
        '#value' => ($row['method'] == 'http' || $row['method'] == 'curl') ?
        l($row['address'], $row['address'], array(
          'attributes' => array('target' => '_blank', 'title' => $last_checked),
        )) :
        $row['address'],
      ),
      'method' => array('#value' => $row['method']),
      'status' => array('#value' => $status[$row['status']]),
      'action' => array('#value' => $action),
      'weight' => array(
        '#type' => 'weight',
        '#delta' => 10,
        '#default_value' => $row['weight'],
        '#attributes' => array('class' => 'weight'),
      ),
    );
    $row_knt++;
  }
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  $form['reset'] = array(
    '#type' => 'button',
    '#value' => t('Cancel'),
  );
  return $form;
}

/**
 * System settings form submit function.
 */
function server_monitor_admin_form_submit($form, &$form_state) {
  switch ($form_state['clicked_button']['#value']) {
    case t('Run monitor'):
      server_monitor_check_all();
      drupal_goto('admin/settings/server_monitor');
      break;

    case t('Save'):
      variable_set('server_monitor_cron', $form_state['values']['server_monitor_cron']);
      variable_set('server_monitor_use_json_block', $form_state['values']['server_monitor_use_json_block']);
      if (isset($form_state['values']['list'])) {
        foreach ($form_state['values']['list'] as $sid => $value) {
          db_query("UPDATE {server_monitor} SET weight=%d WHERE sid=%d", $value['weight'], $sid);
        }
      }
      drupal_set_message('Server Monitor configuration saved.');
      break;

  }
}

/**
 * Implements the Server Monitor server edit page.
 */
function server_monitor_form(&$form_state, $op = NULL, $sid = NULL) {
  $edit = array(
    'name' => '',
    'address' => '',
    'method' => 'http',
    'weight' => 0,
    'server_description' => '',
    'configuration' => array('timeout' => SERVER_MONITOR_DEFAULT_TIMEOUT),
  );
  if ($op == "edit" && $sid) {
    $edit = db_fetch_array(db_query("SELECT * FROM {server_monitor} WHERE sid = %d", $sid));
    $edit['configuration'] = unserialize($edit['configuration']);
    $form['sid'] = array(
      '#type' => 'hidden',
      '#value' => $sid,
    );
  }
  $form['monitor-settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('Server monitor settings'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );
  $form['monitor-settings']['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Server Name'),
    '#default_value' => $edit['name'],
    '#description' => t('Choose a name for this server. This name will be used in reports and blocks.'),
    '#size' => 50,
    '#maxlength' => 255,
    '#required' => TRUE,
  );
  $form['monitor-settings']['address'] = array(
    '#type' => 'textfield',
    '#title' => t('Server Address'),
    '#default_value' => $edit['address'],
    '#description' => t('Enter the full URI or IP address, including the port number (when applicable).'),
    '#size' => 50,
    '#maxlength' => 255,
    '#required' => TRUE,
  );

  $methods = array(
    'http' => t('HTTP Request'),
    'curl' => t('cURL request'),
    'ping' => t('Ping'),
  );

  if (!function_exists('curl_init')) {
    unset($methods['curl']);
  }
  $form['monitor-settings']['method'] = array(
    '#type' => 'select',
    '#title' => t('Monitoring method'),
    '#default_value' => $edit['method'],
    '#options' => $methods,
    '#description' => t('Please select how you would like to check the server status.'),
    '#required' => TRUE,
  );
  $form['monitor-settings']['timeout'] = array(
    '#type' => 'textfield',
    '#title' => t('Timeout'),
    '#default_value' => $edit['configuration']['timeout'],
    '#size' => 3,
    '#maxlength' => 10,
    '#description' => t('Please enter the desired timeout (in seconds).'),
    '#required' => TRUE,
  );
  $form['monitor-settings']['server_description'] = array(
    '#type' => 'textarea',
    '#title' => t('Server Description'),
    '#default_value' => $edit['server_description'],
    '#description' => t('Please enter a brief description of this server.'),
    '#required' => FALSE,
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration'),
  );
  return $form;
}

/**
 * Implements hook_validate().
 */
function server_monitor_form_validate($form, &$form_state) {
  $values = $form_state['values'];
  // Check if server name is already taken.
  if (!isset($values['sid'])) {
    if (db_fetch_object(db_query("SELECT name FROM {server_monitor} WHERE name = '%s'", $values['name']))) {
      form_set_error('name', t('An Server montior with the name %name already exists.', array('%name' => $values['name'])));
    }
  }
  // Check if timeout is valid.
  if (!is_numeric($values['timeout'])) {
    form_set_error('timeout', t('The timeout value must be a number (seconds).'));
  }
}

/**
 * Submit hook for the submit monitor server form.
 */
function server_monitor_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  $configuration = array('timeout' => $values['timeout']);
  if (!isset($values['sid'])) {
    $stmt = "INSERT INTO {server_monitor} (name, address, method, server_description, configuration) VALUES ('%s', '%s', '%s', '%s', '%s')";
    if (db_query($stmt, $values['name'], $values['address'], $values['method'], $values['server_description'], serialize($configuration))) {
      drupal_set_message(t('Server monitor %name has been added.', array('%name' => $values['name'])));
      watchdog('server_monitor', 'Server monitor %name has been added.', array('%name' => $values['name']));
    }
    else {
      drupal_set_message(t('Failed to save new server monitor %name.', array('%name' => $values['name'])), 'error');
      watchdog('server_monitor', 'Failed to save new server monitor %name.', array('%name' => $values['name']), WATCHDOG_ERROR);
    }
  }
  else {
    $stmt = "UPDATE {server_monitor} SET
      name='%s',
      address='%s',
      method='%s',
      server_description='%s',
      configuration='%s' WHERE sid=%d";
    if (db_query($stmt, $values['name'], $values['address'], $values['method'], $values['server_description'], serialize($configuration), $values['sid'])) {
      drupal_set_message(t('Server monitor %name has been successfully modified.', array('%name' => $values['name'])));
      watchdog('server_monitor', 'Server monitor %name has been successfully modified.', array('%name' => $values['name']));
    }
    else {
      drupal_set_message(t('Failed to save server monitor %name.', array('%name' => $values['name'])), 'error');
      watchdog('server_monitor', 'Failed to save server monitor %name.', array('%name' => $values['name']), WATCHDOG_ERROR);
    }
  }
  drupal_goto('admin/settings/server_monitor');
}


/**
 * Implements the Server Monitor server delete page.
 */
function server_monitor_delete_form(&$form_state, $sid = NULL) {
  if (is_numeric($sid) && ($name = db_result(db_query("SELECT name from {server_monitor} WHERE sid = %d", $sid)))) {
    $form = array(
      'sid' => array('#type' => 'hidden', '#value' => $sid),
      'name' => array('#type' => 'hidden', '#value' => $name),
    );
    return confirm_form($form, t('Are you sure you want to delete the server %name?', array('%name' => $name)), 'admin/settings/server_monitor', NULL, t('Delete'), t('Cancel'));
  }
  else {
    drupal_not_found();
    exit;
  }

  return $form;
}


/**
 * Submit hook for the server monitor delete page.
 */
function server_monitor_delete_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  if ($values['confirm'] && $values['sid']) {
    db_query("DELETE FROM {server_monitor} WHERE sid = %d", $values['sid']);
    drupal_set_message(t('Server monitor %name has been deleted.', array('%name' => $values['name'])));
    watchdog('server_monitor', 'Server monitor %name has been deleted.', array('%name' => $values['name']));
  }
  drupal_goto('admin/settings/server_monitor');
}
